@@ -12,10 +12,11 @@
 ** This file contains C code routines that are called by the parser
 ** to handle SELECT statements in SQLite.
 **
-** $Id: select.c,v 1.12 2002/10/16 22:36:04 matt Exp $
+** $Id: select.c,v 1.13 2002/12/18 17:59:18 matt Exp $
 */
 #include "sqliteInt.h"
 
+
 /*
 ** Allocate a new Select structure and return a pointer to that
 ** structure.
@@ -156,7 +157,6 @@ static void addWhereTerm(
 
   dummy.z = zCol;
   dummy.n = strlen(zCol);
-  dummy.base = 1;
   dummy.dyn = 0;
   pE1a = sqliteExpr(TK_ID, 0, 0, &dummy);
   pE2a = sqliteExpr(TK_ID, 0, 0, &dummy);
@@ -169,7 +169,7 @@ static void addWhereTerm(
   pE1c = sqliteExpr(TK_DOT, pE1b, pE1a, 0);
   pE2c = sqliteExpr(TK_DOT, pE2b, pE2a, 0);
   pE = sqliteExpr(TK_EQ, pE1c, pE2c, 0);
-  pE->isJoinExpr = 1;
+  ExprSetProperty(pE, EP_FromJoin);
   if( *ppExpr ){
     *ppExpr = sqliteExpr(TK_AND, *ppExpr, pE, 0);
   }else{
@@ -178,15 +178,18 @@ static void addWhereTerm(
 }
 
 /*
-** Set the Expr.isJoinExpr flag on all terms of the given expression.
+** Set the EP_FromJoin property on all terms of the given expression.
 **
-** The Expr.isJoinExpr flag is used at on terms of an expression to tell
+** The EP_FromJoin property is used at on terms of an expression to tell
 ** the LEFT OUTER JOIN processing logic that this term is part of the
-** join restriction and not a part of the more general WHERE clause.
+** join restriction specified in the ON or USING clause and not a part
+** of the more general WHERE clause.  These terms are moved over to the
+** WHERE clause during join processing but we need to remember that they
+** originated in the ON or USING clause.
 */
 static void setJoinExpr(Expr *p){
   while( p ){
-    p->isJoinExpr = 1;
+    ExprSetProperty(p, EP_FromJoin);
     setJoinExpr(p->pLeft);
     p = p->pRight;
   } 
@@ -277,6 +280,60 @@ static int sqliteProcessJoin(Parse *pParse, Select *p){
   return 0;
 }
 
+/*
+** This routine implements a minimal Oracle8 join syntax immulation.
+** The precise oracle8 syntax is not implemented - it is easy enough
+** to get this routine confused.  But this routine does make it possible
+** to write a single SQL statement that does a left outer join in both
+** oracle8 and in SQLite.
+**
+** This routine looks for TK_COLUMN expression nodes that are marked
+** with the EP_Oracle8Join property.  Such nodes are generated by a
+** column name (either "column" or "table.column") that is followed by
+** the special "(+)" operator.  If the table of the column marked with
+** the (+) operator is the second are subsequent table in a join, then
+** that table becomes the left table in a LEFT OUTER JOIN.  The expression
+** that uses that table becomes part of the ON clause for the join.
+**
+** It is important to enphasize that this is not exactly how oracle8
+** works.  But it is close enough so that one can construct queries that
+** will work correctly for both SQLite and Oracle8.
+*/
+static int sqliteOracle8JoinFixup(
+  int base,         /* VDBE cursor number for first table in pSrc */
+  SrcList *pSrc,    /* List of tables being joined */
+  Expr *pWhere      /* The WHERE clause of the SELECT statement */
+){
+  int rc = 0;
+  if( ExprHasProperty(pWhere, EP_Oracle8Join) && pWhere->op==TK_COLUMN ){
+    int idx = pWhere->iTable - base;
+    assert( idx>=0 && idx<pSrc->nSrc );
+    if( idx>0 ){
+      pSrc->a[idx-1].jointype &= ~JT_INNER;
+      pSrc->a[idx-1].jointype |= JT_OUTER|JT_LEFT;
+      return 1;
+    }
+  }
+  if( pWhere->pRight ){
+    rc = sqliteOracle8JoinFixup(base, pSrc, pWhere->pRight);
+  }
+  if( pWhere->pLeft ){
+    rc |= sqliteOracle8JoinFixup(base, pSrc, pWhere->pLeft);
+  }
+  if( pWhere->pList ){
+    int i;
+    ExprList *pList = pWhere->pList;
+    for(i=0; i<pList->nExpr && rc==0; i++){
+      rc |= sqliteOracle8JoinFixup(base, pSrc, pList->a[i].pExpr);
+    }
+  }
+  if( rc==1 && (pWhere->op==TK_AND || pWhere->op==TK_EQ) ){
+    setJoinExpr(pWhere);
+    rc = 0;
+  }
+  return rc;
+}
+
 /*
 ** Delete the given Select structure and all of its substructures.
 */
@@ -663,9 +720,9 @@ static void generateColumnNames(
         zCol = pTab->aCol[iCol].zName;
         zType = pTab->aCol[iCol].zType;
       }
-      if( p->token.z && p->token.z[0] && !showFullNames ){
+      if( p->span.z && p->span.z[0] && !showFullNames ){
         int addr = sqliteVdbeAddOp(v,OP_ColumnName, i, 0);
-        sqliteVdbeChangeP3(v, -1, p->token.z, p->token.n);
+        sqliteVdbeChangeP3(v, -1, p->span.z, p->span.n);
         sqliteVdbeCompressSpace(v, addr);
       }else if( pTabList->nSrc>1 || showFullNames ){
         char *zName = 0;
@@ -681,13 +738,9 @@ static void generateColumnNames(
         sqliteVdbeAddOp(v, OP_ColumnName, i, 0);
         sqliteVdbeChangeP3(v, -1, zCol, 0);
       }
-    }else if( p->token.z && p->token.z[0] && !showFullNames ){
+    }else if( p->span.z && p->span.z[0] ){
       int addr = sqliteVdbeAddOp(v,OP_ColumnName, i, 0);
-      sqliteVdbeChangeP3(v, -1, p->token.z, p->token.n);
-      sqliteVdbeCompressSpace(v, addr);
-    }else if( p->token.z && p->token.z[0] ){
-      int addr = sqliteVdbeAddOp(v,OP_ColumnName, i, 0);
-      sqliteVdbeChangeP3(v, -1, p->token.z, p->token.n);
+      sqliteVdbeChangeP3(v, -1, p->span.z, p->span.n);
       sqliteVdbeCompressSpace(v, addr);
     }else{
       char zName[30];
@@ -724,6 +777,11 @@ static const char *selectOpName(int id){
   return z;
 }
 
+/*
+** Forward declaration
+*/
+static int fillInColumnList(Parse*, Select*);
+
 /*
 ** Given a SELECT statement, generate a Table structure that describes
 ** the result set of that SELECT.
@@ -732,7 +790,6 @@ Table *sqliteResultSetOfSelect(Parse *pParse, char *zTabName, Select *pSelect){
   Table *pTab;
   int i;
   ExprList *pEList;
-  static int fillInColumnList(Parse*, Select*);
 
   if( fillInColumnList(pParse, pSelect) ){
     return 0;
@@ -750,8 +807,8 @@ Table *sqliteResultSetOfSelect(Parse *pParse, char *zTabName, Select *pSelect){
     Expr *p;
     if( pEList->a[i].zName ){
       pTab->aCol[i].zName = sqliteStrDup(pEList->a[i].zName);
-    }else if( (p=pEList->a[i].pExpr)->token.z && p->token.z[0] ){
-      sqliteSetNString(&pTab->aCol[i].zName, p->token.z, p->token.n, 0);
+    }else if( (p=pEList->a[i].pExpr)->span.z && p->span.z[0] ){
+      sqliteSetNString(&pTab->aCol[i].zName, p->span.z, p->span.n, 0);
     }else if( p->op==TK_DOT && p->pRight && p->pRight->token.z &&
            p->pRight->token.z[0] ){
       sqliteSetNString(&pTab->aCol[i].zName, 
@@ -917,7 +974,6 @@ static int fillInColumnList(Parse *pParse, Select *p){
             pRight->token.z = zName;
             pRight->token.n = strlen(zName);
             pRight->token.dyn = 0;
-            pRight->token.base = 1;
             if( zTabName && pTabList->nSrc>1 ){
               pLeft = sqliteExpr(TK_ID, 0, 0, 0);
               pExpr = sqliteExpr(TK_DOT, pLeft, pRight, 0);
@@ -925,13 +981,15 @@ static int fillInColumnList(Parse *pParse, Select *p){
               pLeft->token.z = zTabName;
               pLeft->token.n = strlen(zTabName);
               pLeft->token.dyn = 0;
-              pLeft->token.base = 1;
-              sqliteSetString((char**)&pExpr->token.z, zTabName, ".", zName, 0);
-              pExpr->token.n = strlen(pExpr->token.z);
-              pExpr->token.base = 0;
-              pExpr->token.dyn = 1;
+              sqliteSetString((char**)&pExpr->span.z, zTabName, ".", zName, 0);
+              pExpr->span.n = strlen(pExpr->span.z);
+              pExpr->span.dyn = 1;
+              pExpr->token.z = 0;
+              pExpr->token.n = 0;
+              pExpr->token.dyn = 0;
             }else{
               pExpr = pRight;
+              pExpr->span = pExpr->token;
             }
             pNew = sqliteExprListAppend(pNew, pExpr, 0);
           }
@@ -1286,12 +1344,12 @@ static int multiSelect(Parse *pParse, Select *p, int eDest, int iParm){
 ** to a column in table number iFrom, change that reference to the
 ** same column in table number iTo.
 */
+static void changeTablesInList(ExprList*, int, int);  /* Forward Declaration */
 static void changeTables(Expr *pExpr, int iFrom, int iTo){
   if( pExpr==0 ) return;
   if( pExpr->op==TK_COLUMN && pExpr->iTable==iFrom ){
     pExpr->iTable = iTo;
   }else{
-    static void changeTablesInList(ExprList*, int, int);
     changeTables(pExpr->pLeft, iFrom, iTo);
     changeTables(pExpr->pRight, iFrom, iTo);
     changeTablesInList(pExpr->pList, iFrom, iTo);
@@ -1320,6 +1378,7 @@ static void changeTablesInList(ExprList *pList, int iFrom, int iTo){
 ** changes to pExpr so that it refers directly to the source table
 ** of the subquery rather the result set of the subquery.
 */
+static void substExprList(ExprList*,int,ExprList*,int);  /* Forward Decl */
 static void substExpr(Expr *pExpr, int iTable, ExprList *pEList, int iSub){
   if( pExpr==0 ) return;
   if( pExpr->op==TK_COLUMN && pExpr->iTable==iTable && pExpr->iColumn>=0 ){
@@ -1338,13 +1397,12 @@ static void substExpr(Expr *pExpr, int iTable, ExprList *pEList, int iSub){
     pExpr->iTable = pNew->iTable;
     pExpr->iColumn = pNew->iColumn;
     pExpr->iAgg = pNew->iAgg;
-    pExpr->nFuncName = pNew->nFuncName;
     sqliteTokenCopy(&pExpr->token, &pNew->token);
+    sqliteTokenCopy(&pExpr->span, &pNew->span);
     if( iSub!=iTable ){
       changeTables(pExpr, iSub, iTable);
     }
   }else{
-    static void substExprList(ExprList*,int,ExprList*,int);
     substExpr(pExpr->pLeft, iTable, pEList, iSub);
     substExpr(pExpr->pRight, iTable, pEList, iSub);
     substExprList(pExpr->pList, iTable, pEList, iSub);
@@ -1412,6 +1470,8 @@ substExprList(ExprList *pList, int iTable, ExprList *pEList, int iSub){
 **  (10)  The subquery does not use aggregates or the outer query does not
 **        use LIMIT.
 **
+**  (11)  The subquery and the outer query do not both have ORDER BY clauses.
+**
 ** In this routine, the "p" parameter is a pointer to the outer query.
 ** The subquery is p->pSrc->a[iFrom].  isAgg is true if the outer query
 ** uses aggregates and subqueryIsAgg is true if the subquery uses aggregates.
@@ -1453,6 +1513,7 @@ static int flattenSubquery(
      return 0;
   }
   if( (p->isDistinct || p->nLimit>=0) && subqueryIsAgg ) return 0;
+  if( p->pOrderBy && pSub->pOrderBy ) return 0;
 
   /* If we reach this point, it means flattening is permitted for the
   ** i-th entry of the FROM clause in the outer query.
@@ -1462,17 +1523,23 @@ static int flattenSubquery(
   substExprList(p->pEList, iParent, pSub->pEList, iSub);
   pList = p->pEList;
   for(i=0; i<pList->nExpr; i++){
-    if( pList->a[i].zName==0 ){
-      Expr *pExpr = pList->a[i].pExpr;
-      assert( pExpr->token.z!=0 );
-      pList->a[i].zName = sqliteStrNDup(pExpr->token.z, pExpr->token.n);
+    Expr *pExpr;
+    if( pList->a[i].zName==0 && (pExpr = pList->a[i].pExpr)->span.z!=0 ){
+      pList->a[i].zName = sqliteStrNDup(pExpr->span.z, pExpr->span.n);
     }
   }
   if( isAgg ){
     substExprList(p->pGroupBy, iParent, pSub->pEList, iSub);
     substExpr(p->pHaving, iParent, pSub->pEList, iSub);
   }
-  substExprList(p->pOrderBy, iParent, pSub->pEList, iSub);
+  if( pSub->pOrderBy ){
+    assert( p->pOrderBy==0 );
+    p->pOrderBy = pSub->pOrderBy;
+    pSub->pOrderBy = 0;
+    changeTablesInList(p->pOrderBy, iSub, iParent);
+  }else if( p->pOrderBy ){
+    substExprList(p->pOrderBy, iParent, pSub->pEList, iSub);
+  }
   if( pSub->pWhere ){
     pWhere = sqliteExprDup(pSub->pWhere);
     if( iParent!=iSub ){
@@ -1588,7 +1655,7 @@ static int simpleMinMaxQuery(Parse *pParse, Select *p, int eDest, int iParm){
   pExpr = p->pEList->a[0].pExpr;
   if( pExpr->op!=TK_AGG_FUNCTION ) return 0;
   if( pExpr->pList==0 || pExpr->pList->nExpr!=1 ) return 0;
-  if( pExpr->nFuncName!=3 ) return 0;
+  if( pExpr->token.n!=3 ) return 0;
   if( sqliteStrNICmp(pExpr->token.z,"min",3)==0 ){
     seekOp = OP_Rewind;
   }else if( sqliteStrNICmp(pExpr->token.z,"max",3)==0 ){
@@ -1801,6 +1868,21 @@ int sqliteSelect(
     if( sqliteExprCheck(pParse, pWhere, 0, 0) ){
       goto select_end;
     }
+    sqliteOracle8JoinFixup(base, pTabList, pWhere);
+  }
+  if( pHaving ){
+    if( pGroupBy==0 ){
+      sqliteSetString(&pParse->zErrMsg, "a GROUP BY clause is required "
+         "before HAVING", 0);
+      pParse->nErr++;
+      goto select_end;
+    }
+    if( sqliteExprResolveIds(pParse, base, pTabList, pEList, pHaving) ){
+      goto select_end;
+    }
+    if( sqliteExprCheck(pParse, pHaving, 1, &isAgg) ){
+      goto select_end;
+    }
   }
   if( pOrderBy ){
     for(i=0; i<pOrderBy->nExpr; i++){
@@ -1848,20 +1930,6 @@ int sqliteSelect(
       }
     }
   }
-  if( pHaving ){
-    if( pGroupBy==0 ){
-      sqliteSetString(&pParse->zErrMsg, "a GROUP BY clause is required "
-         "before HAVING", 0);
-      pParse->nErr++;
-      goto select_end;
-    }
-    if( sqliteExprResolveIds(pParse, base, pTabList, pEList, pHaving) ){
-      goto select_end;
-    }
-    if( sqliteExprCheck(pParse, pHaving, isAgg, 0) ){
-      goto select_end;
-    }
-  }
 
   /* Check for the special case of a min() or max() function by itself
   ** in the result set.
