@@ -15,7 +15,7 @@
 ** individual tokens and sends those tokens one-by-one over to the
 ** parser for analysis.
 **
-** $Id: tokenize.c,v 1.13 2002/10/16 22:36:04 matt Exp $
+** $Id: tokenize.c,v 1.14 2002/12/18 17:59:18 matt Exp $
 */
 #include "sqliteInt.h"
 #include "os.h"
@@ -234,8 +234,13 @@ static int sqliteGetToken(const unsigned char *z, int *tokenType){
       return 1;
     }
     case '(': {
-      *tokenType = TK_LP;
-      return 1;
+      if( z[1]=='+' && z[2]==')' ){
+        *tokenType = TK_ORACLE_OUTER_JOIN;
+        return 3;
+      }else{
+        *tokenType = TK_LP;
+        return 1;
+      }
     }
     case ')': {
       *tokenType = TK_RP;
@@ -424,7 +429,6 @@ int sqliteRunParser(Parse *pParse, const char *zSql, char **pzErrMsg){
       break;
     }
     pParse->sLastToken.z = &zSql[i];
-    pParse->sLastToken.base = 1;
     pParse->sLastToken.dyn = 0;
     pParse->sLastToken.n = sqliteGetToken((unsigned char*)&zSql[i], &tokenType);
     i += pParse->sLastToken.n;
